#_______________________________________________
#
#	SETTINGS
#_______________________________________________
#

#	2DO: Move settings to .INI (or similar)
#	What folder, containing 'projects', should be backed-up?
	$WatchFolder = "c:\develop"

#	Where should these backups land?
	$Repository = "C:\temp\develop"

#_______________________________________________
#
#	INIT
#_______________________________________________
#

# 2DO: unset all variables
# 2DO: check for Everything.exe

#	Make the script folder the working directory
#	(easier to find files/programs that way)

	set-location $PSScriptRoot

	
#	Clean up $WatchFolder and $Repository directory name
#	(for calculating length of foldername later on)
#	Don't trim \ at the front, because of \\server\share or \\?\C:...

	$WatchFolder = $WatchFolder.TrimEnd("\`" ").Trim("`"")
	$Repository = $Repository.TrimEnd("\`" ").Trim("`"")


#	Load Visual Basic 'module'
#	Needed for showing GUI elements (InputBox, MsgBox)

	$null = [Reflection.Assembly]::LoadWithPartialName('Microsoft.VisualBasic')


#	Load Forms 'module'
#	Needed for Browse for folder dialog

	$null = [System.Reflection.Assembly]::LoadWithPartialName("System.windows.forms")

	
#	Delete existing temp folder (%temp%\snap-out)
#	+ Create a new one
#	2DO: check it's existance
#	The foldername is fixed, as it is also used by 2FileList.cmd
#
#	%temp% is the reason we need our own Everything.exe: an installed version 
#	might 'run as administrator' and thus have a different %temp%

	$TempSnap = $Env:temp + "\snap-out"
	If (Test-Path $TempSnap) { rd $TempSnap -Force -Recurse}
	$null = md $TempSnap
	

#_______________________________________________
#
#	GO!
#_______________________________________________
#

#	Ask which project to restore
	
	$ThisProject = gci $Repository -dir | select Name, LastWriteTime | ogv  -Outputmode Single -Title "Project to restore"
	$ThisProject = $ThisProject.Name

	
#	Ask for timestamp snapshot
	
	$CSV = $Repository + "\" + $ThisProject + "\Index\descriptions.csv"
	$TimeStamp = Import-csv $CSV | sort Folder -Descending | ogv  -Outputmode Single -Title "$ThisProject : Select snapshot" 
	$TimeStamp = $TimeStamp.Folder

	
#	Get the right .EFU
	
	$ThisEFU = $Repository + "\" + $ThisProject + "\Index\" + $TimeStamp + ".efu"

	
#	Open this EFU in Everything

	Start -Wait -FilePath .\Everything.exe -ArgumentList "-instance Snap-Out -config .\everything-Snap-Out.ini `"$ThisEFU`" "


#	Now select some files to restore in Everything and press <Enter>
#	That will kick off some CMD's that export the selected filenames
#	(see 2FileList.cmd)
#	Luckily,  while the cmd's are still running, the parent (everything) is still alive!!!
#	( -wait parameter in Start-Process)
#
#	If they are all gone, there are one or more <number>.ff files in %Temp%\snap-out.
#	Put those together to get a complete filelist to copy
#	??? Clean up $TempSnap afterwards?

	$FileList = (cat $TempSnap\*.ff)


#	Select Folder to restore the files in ( default: $WatchFolder )	

    $foldername = New-Object System.Windows.Forms.FolderBrowserDialog
	$foldername.RootFolder = 'MyComputer'
	$foldername.SelectedPath = $WatchFolder
	$foldername.Description = 'In which folder should the files be restored?'
    $Button = $foldername.ShowDialog()
	if ( $Button -eq 'OK' ) {
		$CopyToFolder = $foldername.SelectedPath
	}
#	2DO: MUCH better handkling required

	
#	For each of those files: search for it in the repository.
#	Ignore folders that are newer than the selected $Timestamp

	$RepositoryProject = $Repository + "\" + $ThisProject
	$FileList | % {
		$Found = $false

		# "Cut the tree"
		$Branch = $_.Trim("`"")
		$Branch = $Branch.Substring($WatchFolder.Length + $ThisProject.Length + 1)

		gci $RepositoryProject -dir | Where Name -le $TimeStamp | Sort Name -Descending | % {
			if ($Found) {return}
			$BackupFile = $_.FullName + $Branch
			if (Test-Path $BackupFile) {
				# We found our file. Copy it and go to the next file
				$RestoreFile = $CopyToFolder + $Branch
				# Create folder if needed
				If (-not (Test-Path $RestoreFile\.. )) {$null = md $RestoreFile\.. }
				
				# Copy the file
				copy $BackupFile  $RestoreFile -Force
				$Found = $true
				# 2DO: include errorhandling/-logging
			}
		} # End of this file
	} # End of all files
	
#Done
